import 'dart:convert';

import '../infra/infra.dart';
import 'package:universal_io/io.dart';

class UniversalHttpClient implements HttpDatasource {
  final HttpClient client;

  const UniversalHttpClient(this.client);

  @override
  Future<Response> fetch(Request unoRequest) async {
    client.connectionTimeout = unoRequest.timeout;
    final request = await client.openUrl(unoRequest.method, unoRequest.uri);
    request.headers.set('content-length', unoRequest.bodyBytes.length.toString());

    for (var key in unoRequest.headers.keys) {
      request.headers.set(key, unoRequest.headers[key]!);
    }

    request.add(unoRequest.bodyBytes);
    final response = await request.close();
    final buffer = StringBuffer();
    await for (var item in response.transform(utf8.decoder)) {
      buffer.write(item);
    }

    final json = jsonDecode(buffer.toString());

    final headers = <String, String>{};

    response.headers.forEach((key, values) {
      headers[key] = values.join(',');
    });

    final unoResponse = Response(
      request: unoRequest,
      status: response.statusCode,
      data: json,
      headers: headers,
    );
    return unoResponse;
  }
}
